import * as d3 from "d3";
import $ from "jquery";

function logjam_heatmap_plot(params) {
  var data      = params.data,
      interval  = params.interval,
      container = params.container,
      resource  = params.resource;

  var B = params.data["0"].length;

  function get_height() {
    var enlarged_size = $('#enlarged-plot').height() - 130;
    if (enlarged_size > 0) {
      return enlarged_size;
    }
    var parent_height = $(container).parent('.item').height() - 100;
    return parent_height > 170 ? parent_height : 170;
  }

  var w = (document.getElementById(container.slice(1)).offsetWidth - 60 < 400) ? 626 : document.getElementById(container.slice(1)).offsetWidth - 60,
      h = get_height(),
      xticks = d3.range(25).map((h) => h/interval*60),
      x      = d3.scaleLinear().domain([0, 1440/interval]).range([0, w]),
      y      = d3.scaleLinear().domain([0, B]).range([h, 0]).nice(),
      cardWidth = x(2)-x(1)+1,
      cardHeight = h/B;

  /* tiles */
  var tiles = [];
  for (var key in data)
    tiles = tiles.concat(data[key].map((d,i) => {
      return {
        minute: +key,
        bucket: i,
        value: d
      };
    }).filter((e) => e.value > 0));

  var origMaxValue = d3.max(tiles, (d) => d.value);

  if (params.scale == 'logarithmic') {
    tiles.forEach((d) => {
      d.value = Math.log(d.value);
    });
  };

  // console.log(tiles);

  var maxValue = d3.max(tiles, (d) => d.value);
  var maxBucket = d3.max(tiles, (d) => d.bucket);

  //d3.interpolateRgb.gamma(3)("rgb(168, 204, 255)", "rgb(0,0,255)"))

  var colorInterpolator;
  if (resource == 'page_time') {
    colorInterpolator = d3.interpolateGreens;
  } else if (resource == 'ajax_time') {
    colorInterpolator = d3.interpolateReds;
  } else {
    colorInterpolator = d3.interpolateBlues;
  }
  // by not using the full scale, we make low number of requests more visible
  var color = d3.scaleSequential(colorInterpolator)
      .domain([-0.1 * maxValue, maxValue]);

  /* The root panel. */
  var vis = d3.select(params.container)
        .append("svg")
        .attr("width", w+50)
        .attr("height", h+80)
        .style("stroke", "#999")
        .style("strokeWidth", 1.0)
        .append("g")
        .attr("transform", "translate(40,30)");

  /* X-label */
  vis.append("svg:text")
    .attr("class", "label")
    .attr("dy", h+30)
    .attr("dx", w/2)
    .style("font", "12px Helvetica Neue")
    .attr("text-anchor", "middle")
    .text("Time of day");

  /* X-axis and ticks. */
  vis.append("line")
    .attr("class", "xrule")
    .style("stroke", "#999")
    .attr("y1", h)
    .attr("x1", 0)
    .attr("y2", h)
    .attr("x2", w);

  vis.selectAll(".xlabel")
    .data(xticks)
    .enter().append("text")
    .attr("class", "xlabel")
    .attr("x", x)
    .attr("y", h)
    .attr("dx", 0)
    .attr("dy", 12)
    .attr("text-anchor", "middle")
    .style("font", "8px Helvetica Neue")
    .text((d) => (d*interval)/60);

  /* Y-label */
  vis.append("svg:text")
    .attr("class", "label")
    .attr("dy", -30)
    .attr("dx", -h/2)
    .style("font", "12px Helvetica Neue")
    .attr("text-anchor", "middle")
    .attr("transform", "rotate(270)")
    .text("Response Time");

  /* Y-axis and ticks. */
  vis.append("line")
    .attr("class", "yrule")
    .style("stroke", "#999")
    .attr("y1", h)
    .attr("x1", 0)
    .attr("y2", 0)
    .attr("x2", 0);

  var ylabels = ["0ms", "1ms", "3ms", "10ms", "30ms", "0.1s", "0.3s", "1s", "3s", "10s", "30s", "100s",
                 "5m", "17m", "50m", "2.6h", "8.3h", "1.2d", "3.5d"];

  vis.selectAll(".ylabel")
    .data(y.ticks(maxBucket))
    .enter()
    .append("text")
      .attr("class", "ylabel")
      .attr("x", 0)
      .attr("y", y)
      .attr("dx", -3)
      .attr("dy", 0)
      .attr("text-anchor", "end")
      .style("font", "8px Helvetica Neue")
      .text((d,i) => ylabels[i]);

  var cards = vis.selectAll(".card")
       .data(tiles, (d) => d.minute+':'+d.bucket);

  var tooltip_text = "";
  var tooltip_formatter = d3.format(",d");

  function format(value) {
    if (params.scale == 'logarithmic')
      value = Math.pow(Math.E, value);
    return tooltip_formatter(value) + " requests";
  }

  cards.append("title");

  cards.enter().append("rect")
    .attr("x", (d) => x(d.minute))
    .attr("y", (d) => y(d.bucket+1))
    .attr("class", "card")
    .attr("width", cardWidth)
    .attr("height", cardHeight)
    .on("mousemove", function(e, d){ tooltip_text = format(d.value);})
    .on("mouseover", function(e, d){ tooltip_text = format(d.value);})
    .on("mouseout", function(e, d){ tooltip_text = ""; })
    .style("stroke-width", 0)
    .style("fill", function(d){ return color(d.value);});

  cards.transition().duration(1000)
    .style("fill", function(d) { return color(d.value); });

  cards.select("title").text(function(d) { return d.value; });

  cards.exit().remove();

  $('.card').tipsy({
    trigger: 'hover',
    follow: 'x',
    offset: 0,
    offsetX: 0,
    offsetY: -20,
    gravity: 's',
    html: false,
    title: function() { return tooltip_text; }
  });

  // Legend
  var legendWidth = w/2-70-1;

  var legendScale = d3.scaleLinear()
      .domain([1, origMaxValue])
      .range([0, legendWidth-1]);

  var legend = d3.axisBottom()
      .scale(legendScale)
      .tickArguments([10, "s"]);

  vis.append("g")
    .attr("transform", "translate(0,"+(h+35)+")")
    .call(legend);

  vis.selectAll(".mybars")
    .data(d3.range(legendWidth))
    .enter().append("rect")
    .attr("class", "mybars")
    .attr("x", function(d, i) { return i; })
    .attr("y", h+20)
    .attr("height", 15)
    .attr("width", 1)
    .style("stroke", "none")
    .style("fill", function(d, i ) { return color((maxValue*1.0/legendWidth)*i); });
}

window.logjam_heatmap_plot = logjam_heatmap_plot;
